@@ -37,7 +37,7 @@
 #include "gtksourceundomanager.h"
 #include "gtksourceview-marshal.h"
 #include "gtksourceiter.h"
-#include "gtksourcestylemanager.h"
+#include "gtksourcestyleschememanager.h"
 #include "gtksourcestyle-private.h"
 
 /*
@@ -73,8 +73,8 @@ enum {
 	PROP_0,
 	PROP_CAN_UNDO,
 	PROP_CAN_REDO,
-	PROP_CHECK_BRACKETS,
-	PROP_HIGHLIGHT,
+	PROP_HIGHLIGHT_SYNTAX,
+	PROP_HIGHLIGHT_MATCHING_BRACKETS,
 	PROP_MAX_UNDO_LEVELS,
 	PROP_LANGUAGE,
 	PROP_STYLE_SCHEME
@@ -82,8 +82,8 @@ enum {
 
 struct _GtkSourceBufferPrivate
 {
-	gint                   highlight:1;
-	gint                   check_brackets:1;
+	gint                   highlight_syntax:1;
+	gint                   highlight_brackets:1;
 
 	GtkTextTag            *bracket_match_tag;
 	GtkTextMark           *bracket_mark;
@@ -137,8 +137,6 @@ static void 	 gtk_source_buffer_real_delete_range 	(GtkTextBuffer           *buf
 
 static gboolean	 gtk_source_buffer_find_bracket_match_with_limit (GtkTextIter    *orig,
 								  gint            max_chars);
-static void      update_bracket_match_style             (GtkSourceBuffer *source_buffer);
-
 
 static void
 gtk_source_buffer_class_init (GtkSourceBufferClass *klass)
@@ -160,38 +158,54 @@ gtk_source_buffer_class_init (GtkSourceBufferClass *klass)
 	tb_class->insert_text 	= gtk_source_buffer_real_insert_text;
 	tb_class->delete_range 	= gtk_source_buffer_real_delete_range;
 
+	/**
+	 * GtkSourceBuffer:highlight-syntax:
+	 *
+	 * Whether to highlight syntax in the buffer.
+	 */
 	g_object_class_install_property (object_class,
-					 PROP_CHECK_BRACKETS,
-					 g_param_spec_boolean ("check_brackets",
-							       _("Check Brackets"),
-							       _("Whether to check and "
-								 "highlight matching brackets"),
-							       TRUE,
-							       G_PARAM_READWRITE));
-
-	g_object_class_install_property (object_class,
-					 PROP_HIGHLIGHT,
-					 g_param_spec_boolean ("highlight",
-							       _("Highlight"),
+					 PROP_HIGHLIGHT_SYNTAX,
+					 g_param_spec_boolean ("highlight-syntax",
+							       _("Highlight Syntax"),
 							       _("Whether to highlight syntax "
 								 "in the buffer"),
 							       TRUE,
 							       G_PARAM_READWRITE));
 
+	/**
+	 * GtkSourceBuffer:highlight-matching-brackets:
+	 *
+	 * Whether to highlight matching brackets in the buffer.
+	 */
+	g_object_class_install_property (object_class,
+					 PROP_HIGHLIGHT_MATCHING_BRACKETS,
+					 g_param_spec_boolean ("highlight-matching-brackets",
+							       _("Highlight Matching Brackets"),
+							       _("Whether to highlight matching brackets"),
+							       TRUE,
+							       G_PARAM_READWRITE));
+
+	/**
+	 * GtkSourceBuffer:max-undo-levels:
+	 *
+	 * Number of undo levels for the buffer. -1 means no limit.
+	 */
 	g_object_class_install_property (object_class,
 					 PROP_MAX_UNDO_LEVELS,
-					 g_param_spec_int ("max_undo_levels",
+					 g_param_spec_int ("max-undo-levels",
 							   _("Maximum Undo Levels"),
 							   _("Number of undo levels for "
 							     "the buffer"),
-							   0,
-							   200,
-							   25,
+							   -1,
+							   G_MAXINT,
+							   1000,
 							   G_PARAM_READWRITE));
 
 	g_object_class_install_property (object_class,
 					 PROP_LANGUAGE,
 					 g_param_spec_object ("language",
+							      /* Translators: throughout gtksourceview "language" stands
+							       * for "programming language", not "spoken language" */
 							      _("Language"),
 							      _("Language object to get "
 								"highlighting patterns from"),
@@ -220,8 +234,6 @@ gtk_source_buffer_class_init (GtkSourceBufferClass *klass)
 	 * Style scheme. It contains styles for syntax highlighting, optionally
 	 * foreground, background, cursor color, current line color, and matching
 	 * brackets style.
-	 *
-	 * Since: 2.0
 	 */
 	g_object_class_install_property (object_class,
 					 PROP_STYLE_SCHEME,
@@ -269,28 +281,26 @@ gtk_source_buffer_init (GtkSourceBuffer *buffer)
 
 	priv->undo_manager = gtk_source_undo_manager_new (GTK_TEXT_BUFFER (buffer));
 
-	priv->check_brackets = TRUE;
+	priv->highlight_syntax = TRUE;
+	priv->highlight_brackets = TRUE;
 	priv->bracket_mark = NULL;
 	priv->bracket_found = FALSE;
 
 	priv->markers = g_array_new (FALSE, FALSE, sizeof (GtkSourceMarker *));
 
-	priv->highlight = TRUE;
 	priv->style_scheme = _gtk_source_style_scheme_get_default ();
 	if (priv->style_scheme != NULL)
 		g_object_ref (priv->style_scheme);
 
-	g_signal_connect (G_OBJECT (buffer),
+	g_signal_connect (buffer,
 			  "mark_set",
 			  G_CALLBACK (gtk_source_buffer_move_cursor),
 			  NULL);
-
-	g_signal_connect (G_OBJECT (priv->undo_manager),
+	g_signal_connect (priv->undo_manager,
 			  "can_undo",
 			  G_CALLBACK (gtk_source_buffer_can_undo_handler),
 			  buffer);
-
-	g_signal_connect (G_OBJECT (priv->undo_manager),
+	g_signal_connect (priv->undo_manager,
 			  "can_redo",
 			  G_CALLBACK (gtk_source_buffer_can_redo_handler),
 			  buffer);
@@ -366,14 +376,14 @@ gtk_source_buffer_set_property (GObject      *object,
 
 	switch (prop_id)
 	{
-		case PROP_CHECK_BRACKETS:
-			gtk_source_buffer_set_check_brackets (source_buffer,
+		case PROP_HIGHLIGHT_SYNTAX:
+			gtk_source_buffer_set_highlight_syntax (source_buffer,
 							      g_value_get_boolean (value));
 			break;
 
-		case PROP_HIGHLIGHT:
-			gtk_source_buffer_set_highlight (source_buffer,
-							 g_value_get_boolean (value));
+		case PROP_HIGHLIGHT_MATCHING_BRACKETS:
+			gtk_source_buffer_set_highlight_matching_brackets (source_buffer,
+								g_value_get_boolean (value));
 			break;
 
 		case PROP_MAX_UNDO_LEVELS:
@@ -411,12 +421,14 @@ gtk_source_buffer_get_property (GObject    *object,
 
 	switch (prop_id)
 	{
-		case PROP_CHECK_BRACKETS:
-			g_value_set_boolean (value, source_buffer->priv->check_brackets);
+		case PROP_HIGHLIGHT_SYNTAX:
+			g_value_set_boolean (value,
+					     source_buffer->priv->highlight_syntax);
 			break;
 
-		case PROP_HIGHLIGHT:
-			g_value_set_boolean (value, source_buffer->priv->highlight);
+		case PROP_HIGHLIGHT_MATCHING_BRACKETS:
+			g_value_set_boolean (value,
+					     source_buffer->priv->highlight_brackets);
 			break;
 
 		case PROP_MAX_UNDO_LEVELS:
@@ -448,7 +460,7 @@ gtk_source_buffer_get_property (GObject    *object,
 
 /**
  * gtk_source_buffer_new:
- * @table: a #GtkSourceTagTable, or %NULL to create a new one.
+ * @table: a #GtkTextTagTable, or %NULL to create a new one.
  *
  * Creates a new source buffer.
  *
@@ -468,11 +480,10 @@ gtk_source_buffer_new (GtkTextTagTable *table)
  *
  * Creates a new source buffer using the highlighting patterns in
  * @language.  This is equivalent to creating a new source buffer with
- * the default tag table and then calling
- * gtk_source_buffer_set_language().
+ * a new tag table and then calling gtk_source_buffer_set_language().
  *
  * Return value: a new source buffer which will highlight text
- * according to @language.
+ * according to the highlighting patterns in @language.
  **/
 GtkSourceBuffer *
 gtk_source_buffer_new_with_language (GtkSourceLanguage *language)
@@ -508,6 +519,20 @@ gtk_source_buffer_can_redo_handler (GtkSourceUndoManager  	*um,
 	g_object_notify (G_OBJECT (buffer), "can-redo");
 }
 
+static void
+update_bracket_match_style (GtkSourceBuffer *buffer)
+{
+	if (buffer->priv->bracket_match_tag != NULL)
+	{
+		GtkSourceStyle *style = NULL;
+
+		if (buffer->priv->style_scheme)
+			style = _gtk_source_style_scheme_get_matching_brackets_style (buffer->priv->style_scheme);
+
+		_gtk_source_style_apply (style, buffer->priv->bracket_match_tag);
+	}
+}
+
 static GtkTextTag *
 get_bracket_match_tag (GtkSourceBuffer *buffer)
 {
@@ -552,7 +577,7 @@ gtk_source_buffer_move_cursor (GtkTextBuffer     *buffer,
 					    &iter2);
 	}
 
-	if (!GTK_SOURCE_BUFFER (buffer)->priv->check_brackets)
+	if (!GTK_SOURCE_BUFFER (buffer)->priv->highlight_brackets)
 		return;
 
 	iter1 = *iter;
@@ -940,7 +965,8 @@ gtk_source_buffer_redo (GtkSourceBuffer *buffer)
  * Determines the number of undo levels the buffer will track for
  * buffer edits.
  *
- * Return value: the maximum number of possible undo levels.
+ * Return value: the maximum number of possible undo levels or
+ *               -1 if no limit is set.
  **/
 gint
 gtk_source_buffer_get_max_undo_levels (GtkSourceBuffer *buffer)
@@ -959,6 +985,8 @@ gtk_source_buffer_get_max_undo_levels (GtkSourceBuffer *buffer)
  * track.  If the number of user actions exceeds the limit set by this
  * function, older actions will be discarded.
  *
+ * If @max_undo_levels is -1, no limit is set.
+ *
  * A new action is started whenever the function
  * gtk_text_buffer_begin_user_action() is called.  In general, this
  * happens whenever the user presses any key which modifies the
@@ -977,7 +1005,7 @@ gtk_source_buffer_set_max_undo_levels (GtkSourceBuffer *buffer,
 	{
 		gtk_source_undo_manager_set_max_undo_levels (buffer->priv->undo_manager,
 							     max_undo_levels);
-		g_object_notify (G_OBJECT (buffer), "max_undo_levels");
+		g_object_notify (G_OBJECT (buffer), "max-undo-levels");
 	}
 }
 
@@ -1019,7 +1047,7 @@ gtk_source_buffer_end_not_undoable_action (GtkSourceBuffer *buffer)
 }
 
 /**
- * gtk_source_buffer_get_check_brackets:
+ * gtk_source_buffer_get_highlight_matching_brackets:
  * @buffer: a #GtkSourceBuffer.
  *
  * Determines whether bracket match highlighting is activated for the
@@ -1029,17 +1057,17 @@ gtk_source_buffer_end_not_undoable_action (GtkSourceBuffer *buffer)
  * brackets.
  **/
 gboolean
-gtk_source_buffer_get_check_brackets (GtkSourceBuffer *buffer)
+gtk_source_buffer_get_highlight_matching_brackets (GtkSourceBuffer *buffer)
 {
 	g_return_val_if_fail (GTK_IS_SOURCE_BUFFER (buffer), FALSE);
 
-	return buffer->priv->check_brackets;
+	return buffer->priv->highlight_brackets;
 }
 
 /**
- * gtk_source_buffer_set_check_brackets:
+ * gtk_source_buffer_set_highlight_matching_brackets:
  * @buffer: a #GtkSourceBuffer.
- * @check_brackets: %TRUE if you want matching brackets highlighted.
+ * @highlight: %TRUE if you want matching brackets highlighted.
  *
  * Controls the bracket match highlighting function in the buffer.  If
  * activated, when you position your cursor over a bracket character
@@ -1049,92 +1077,76 @@ gtk_source_buffer_get_check_brackets (GtkSourceBuffer *buffer)
  * function.
  **/
 void
-gtk_source_buffer_set_check_brackets (GtkSourceBuffer *buffer,
-				      gboolean         check_brackets)
-{
-	g_return_if_fail (GTK_IS_SOURCE_BUFFER (buffer));
-
-	check_brackets = (check_brackets != FALSE);
-
-	if (check_brackets != buffer->priv->check_brackets)
-	{
-		buffer->priv->check_brackets = check_brackets;
-		g_object_notify (G_OBJECT (buffer), "check_brackets");
-	}
-}
-
-static void
-update_bracket_match_style (GtkSourceBuffer *buffer)
-{
-	if (buffer->priv->bracket_match_tag != NULL)
-	{
-		GtkSourceStyle *style = NULL;
-
-		if (buffer->priv->style_scheme)
-			style = gtk_source_style_scheme_get_matching_brackets_style (buffer->priv->style_scheme);
-
-		_gtk_source_style_apply (style, buffer->priv->bracket_match_tag);
-	}
-}
-
-/**
- * gtk_source_buffer_get_highlight:
- * @buffer: a #GtkSourceBuffer.
- *
- * Determines whether text highlighting is activated in the source
- * buffer.
- *
- * Return value: %TRUE if highlighting is enabled.
- **/
-gboolean
-gtk_source_buffer_get_highlight (GtkSourceBuffer *buffer)
-{
-	g_return_val_if_fail (GTK_IS_SOURCE_BUFFER (buffer), FALSE);
-
-	return buffer->priv->highlight;
-}
-
-/**
- * gtk_source_buffer_set_highlight:
- * @buffer: a #GtkSourceBuffer.
- * @highlight: %TRUE if you want to activate highlighting.
- *
- * Controls whether text is highlighted in the buffer.  If @highlight
- * is %TRUE, the text will be highlighted according to the syntax
- * highlighting specification installed in the buffer with
- * gtk_source_buffer_set_language(). Otherwise, any current highlighted
- * text will be restored to the default buffer style.
- *
- * Tags not of #GtkSourceTag type will not be removed by this option,
- * and normal #GtkTextTag priority settings apply when highlighting is
- * enabled.
- **/
-void
-gtk_source_buffer_set_highlight (GtkSourceBuffer *buffer,
-				 gboolean         highlight)
+gtk_source_buffer_set_highlight_matching_brackets (GtkSourceBuffer *buffer,
+						   gboolean         highlight)
 {
 	g_return_if_fail (GTK_IS_SOURCE_BUFFER (buffer));
 
 	highlight = (highlight != FALSE);
 
-	if (buffer->priv->highlight == highlight)
-		return;
-
-	buffer->priv->highlight = highlight;
-
-	g_object_notify (G_OBJECT (buffer), "highlight");
+	if (highlight != buffer->priv->highlight_brackets)
+	{
+		buffer->priv->highlight_brackets = highlight;
+		g_object_notify (G_OBJECT (buffer), "highlight-matching-brackets");
+	}
 }
 
+/**
+ * gtk_source_buffer_get_highlight_syntax:
+ * @buffer: a #GtkSourceBuffer.
+ *
+ * Determines whether syntax highlighting is activated in the source
+ * buffer.
+ *
+ * Return value: %TRUE if syntax highlighting is enabled, %FALSE otherwise.
+ **/
+gboolean
+gtk_source_buffer_get_highlight_syntax (GtkSourceBuffer *buffer)
+{
+	g_return_val_if_fail (GTK_IS_SOURCE_BUFFER (buffer), FALSE);
+
+	return buffer->priv->highlight_syntax;
+}
+
+/**
+ * gtk_source_buffer_set_highlight_syntax:
+ * @buffer: a #GtkSourceBuffer.
+ * @highlight: %TRUE to enable syntax highlighting, %FALSE to disable it.
+ *
+ * Controls whether syntax is highlighted in the buffer. If @highlight
+ * is %TRUE, the text will be highlighted according to the syntax
+ * patterns specified in the language set with
+ * gtk_source_buffer_set_language(). If @highlight is %FALSE, syntax highlighting
+ * is disabled and all the GtkTextTag objects that have been added by the 
+ * syntax highlighting engine are removed from the buffer.
+ **/
+void
+gtk_source_buffer_set_highlight_syntax (GtkSourceBuffer *buffer,
+					gboolean         highlight)
+{
+	g_return_if_fail (GTK_IS_SOURCE_BUFFER (buffer));
+
+	highlight = (highlight != FALSE);
+
+	if (buffer->priv->highlight_syntax != highlight)
+	{
+	buffer->priv->highlight_syntax = highlight;
+	g_object_notify (G_OBJECT (buffer), "highlight-syntax");
+	}
+}
 
 /**
  * gtk_source_buffer_set_language:
  * @buffer: a #GtkSourceBuffer.
  * @language: a #GtkSourceLanguage to set, or %NULL.
  *
- * Sets the #GtkSourceLanguage the source buffer will use, adding
- * #GtkSourceTag tags with the language's patterns. Note that
- * this will remove any #GtkSourceTag tags currently in the buffer's
- * tag table.  The buffer holds a reference to the @language set.
+ * Associate a #GtkSourceLanguage with the source buffer. If @language is 
+ * not-%NULL and syntax highlighting is enabled (see gtk_source_buffer_set_highlight()),
+ * the syntax patterns defined in @language will be used to highlight the text
+ * contained in the buffer. If @language is %NULL, the text contained in the 
+ * buffer is not highlighted.
+ * 
+ * The buffer holds a reference to @language.
  **/
 void
 gtk_source_buffer_set_language (GtkSourceBuffer   *buffer,
@@ -1183,11 +1195,11 @@ gtk_source_buffer_set_language (GtkSourceBuffer   *buffer,
  * gtk_source_buffer_get_language:
  * @buffer: a #GtkSourceBuffer.
  *
- * Determines the #GtkSourceLanguage used by the buffer.  The returned
- * object should not be unreferenced by the user.
+ * Returns the #GtkSourceLanguage associated with the buffer, 
+ * see gtk_source_buffer_set_language().  The returned object should not be
+ * unreferenced by the user.
  *
- * Return value: the #GtkSourceLanguage set by
- * gtk_source_buffer_set_language(), or %NULL.
+ * Return value: #GtkSourceLanguage associated with the buffer, or %NULL.
  **/
 GtkSourceLanguage *
 gtk_source_buffer_get_language (GtkSourceBuffer *buffer)
@@ -1221,6 +1233,33 @@ _gtk_source_buffer_update_highlight (GtkSourceBuffer   *buffer,
 						     synchronous);
 }
 
+/**
+ * gtk_source_buffer_ensure_highlight:
+ * @buffer: a #GtkSourceBuffer.
+ * @start: start of the area to highlight.
+ * @end: end of the area to highlight.
+ *
+ * Forces buffer to analyze and highlight the given area synchronously.
+ *
+ * <note>
+ *   <para>
+ *     This is a potentially slow operation and should be used only
+ *     when you need to make sure that some text not currently
+ *     visible is highlighted, for instance before printing.
+ *   </para>
+ * </note>
+ **/
+void
+gtk_source_buffer_ensure_highlight (GtkSourceBuffer   *buffer,
+				    const GtkTextIter *start,
+				    const GtkTextIter *end)
+{
+	_gtk_source_buffer_update_highlight (buffer,
+					     start,
+					     end,
+					     TRUE);
+}
+
 /**
  * gtk_source_buffer_set_style_scheme:
  * @buffer: a #GtkSourceBuffer.
