@@ -514,7 +514,7 @@ add_ref (ParserState               *parser_state,
 			GtkSourceLanguage *imported_language;
 
 			lm = _gtk_source_language_get_language_manager (parser_state->language);
-			imported_language = gtk_source_language_manager_get_language_by_id (lm, lang_id);
+			imported_language = gtk_source_language_manager_get_language (lm, lang_id);
 
 			if (imported_language == NULL)
 			{
@@ -1268,7 +1268,7 @@ parse_language_with_id (ParserState *parser_state,
 	g_return_if_fail (parser_state->error == NULL);
 
 	lm = _gtk_source_language_get_language_manager (parser_state->language);
-	imported_language = gtk_source_language_manager_get_language_by_id (lm, lang_id);
+	imported_language = gtk_source_language_manager_get_language (lm, lang_id);
 
 	if (imported_language == NULL)
 	{
@@ -1413,19 +1413,18 @@ handle_keyword_char_class_element (ParserState *parser_state)
 static void
 handle_styles_element (ParserState *parser_state)
 {
-	int ret, type;
+	int type;
 	const xmlChar *tag_name;
 
 	g_return_if_fail (parser_state->error == NULL);
 
 	while (parser_state->error == NULL)
 	{
-		ret = xmlTextReaderRead (parser_state->reader);
-
-		xmlTextReaderIsValid (parser_state->reader);
-
 		/* FIXME: is xmlTextReaderIsValid call needed here or
 		 * error func will be called? */
+		xmlTextReaderRead (parser_state->reader);
+		xmlTextReaderIsValid (parser_state->reader);
+
 		if (parser_state->error != NULL)
 			break;
 
