@@ -19,7 +19,7 @@
  */
 
 #include "gtksourceview-i18n.h"
-#include "gtksourcestylemanager.h"
+#include "gtksourcestyleschememanager.h"
 #include "gtksourceview.h"
 #include "gtksourcelanguage-private.h"
 #include "gtksourcestyle-private.h"
@@ -45,6 +45,7 @@ enum {
 	PROP_0,
 	PROP_ID,
 	PROP_NAME,
+	PROP_DESCRIPTION,
 	PROP_FILENAME
 };
 
@@ -52,7 +53,7 @@ struct _GtkSourceStyleSchemePrivate
 {
 	gchar *id;
 	gchar *name;
-	gchar *author;
+	GPtrArray *authors;
 	gchar *description;
 	gchar *filename;
 	GtkSourceStyleScheme *parent;
@@ -73,7 +74,13 @@ gtk_source_style_scheme_finalize (GObject *object)
 	g_hash_table_destroy (scheme->priv->style_cache);
 	g_hash_table_destroy (scheme->priv->defined_styles);
 	g_free (scheme->priv->filename);
-	g_free (scheme->priv->author);
+	
+	if (scheme->priv->authors != NULL) 
+	{
+		g_ptr_array_foreach (scheme->priv->authors, (GFunc)g_free, NULL);
+		g_ptr_array_free (scheme->priv->authors, TRUE);
+	}
+	
 	g_free (scheme->priv->description);
 	g_free (scheme->priv->id);
 	g_free (scheme->priv->name);
@@ -102,13 +109,6 @@ gtk_source_style_scheme_set_property (GObject 	   *object,
 			g_free (tmp);
 			break;
 
-		case PROP_NAME:
-			tmp = scheme->priv->name;
-			scheme->priv->name = g_value_dup_string (value);
-			g_free (tmp);
-			g_object_notify (object, "name");
-			break;
-
 		default:
 			G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
 			break;
@@ -133,6 +133,10 @@ gtk_source_style_scheme_get_property (GObject 	 *object,
 			g_value_set_string (value, scheme->priv->name);
 			break;
 
+		case PROP_DESCRIPTION:
+			g_value_set_string (value, scheme->priv->description);
+			break;
+
 		case PROP_FILENAME:
 			g_value_set_string (value, scheme->priv->filename);
 			break;
@@ -156,9 +160,7 @@ gtk_source_style_scheme_class_init (GtkSourceStyleSchemeClass *klass)
 	 * GtkSourceStyleScheme:id:
 	 *
 	 * Style scheme id, a unique string used to identify the style scheme
-	 * in #GtkSourceStyleManager.
-	 *
-	 * Since: 2.0
+	 * in #GtkSourceStyleSchemeManager.
 	 */
 	g_object_class_install_property (object_class,
 					 PROP_ID,
@@ -172,8 +174,6 @@ gtk_source_style_scheme_class_init (GtkSourceStyleSchemeClass *klass)
 	 * GtkSourceStyleScheme:name:
 	 *
 	 * Style scheme name, a translatable string to present to user.
-	 *
-	 * Since: 2.0
 	 */
 	g_object_class_install_property (object_class,
 					 PROP_NAME,
@@ -181,14 +181,25 @@ gtk_source_style_scheme_class_init (GtkSourceStyleSchemeClass *klass)
 						 	      _("Style scheme name"),
 							      _("Style scheme name"),
 							      NULL,
-							      G_PARAM_READWRITE));
+							      G_PARAM_READABLE));
+
+	/**
+	 * GtkSourceStyleScheme:name:
+	 *
+	 * Style scheme name, a translatable string to present to user.
+	 */
+	g_object_class_install_property (object_class,
+					 PROP_NAME,
+					 g_param_spec_string ("description",
+						 	      _("Style scheme description"),
+							      _("Style scheme description"),
+							      NULL,
+							      G_PARAM_READABLE));
 
 	/**
 	 * GtkSourceStyleScheme:filename:
 	 *
 	 * Style scheme filename or NULL.
-	 *
-	 * Since: 2.0
 	 */
 	g_object_class_install_property (object_class,
 					 PROP_FILENAME,
@@ -268,6 +279,27 @@ gtk_source_style_scheme_get_description (GtkSourceStyleScheme *scheme)
 	return scheme->priv->description;
 }
 
+/**
+ * gtk_source_style_scheme_get_authors:
+ * @scheme: a #GtkSourceStyleScheme.
+ *
+ * Returns: a %NULL-terminated array containing the @scheme authors or 
+ * %NULL if no author is specified by the style
+ * scheme.
+ *
+ * Since: 2.0
+ */
+G_CONST_RETURN gchar* G_CONST_RETURN *
+gtk_source_style_scheme_get_authors (GtkSourceStyleScheme *scheme)
+{
+	g_return_val_if_fail (GTK_IS_SOURCE_STYLE_SCHEME (scheme), NULL);
+
+	if (scheme->priv->authors == NULL)
+		return NULL;
+
+	return (G_CONST_RETURN gchar* G_CONST_RETURN *)scheme->priv->authors->pdata;
+}
+
 /**
  * gtk_source_style_scheme_get_filename:
  * @scheme: a #GtkSourceStyleScheme.
@@ -358,29 +390,32 @@ fix_style_colors (GtkSourceStyleScheme *scheme,
 		  GtkSourceStyle       *real_style)
 {
 	GtkSourceStyle *style;
+	guint i;
+	struct {
+		guint mask;
+		guint offset;
+	} attributes[] = {
+		{ GTK_SOURCE_STYLE_USE_BACKGROUND, G_STRUCT_OFFSET (GtkSourceStyle, background) },
+		{ GTK_SOURCE_STYLE_USE_FOREGROUND, G_STRUCT_OFFSET (GtkSourceStyle, foreground) },
+		{ GTK_SOURCE_STYLE_USE_LINE_BACKGROUND, G_STRUCT_OFFSET (GtkSourceStyle, line_background) }
+	};
 
 	style = gtk_source_style_copy (real_style);
 
-	if (style->mask & GTK_SOURCE_STYLE_USE_BACKGROUND)
+	for (i = 0; i < G_N_ELEMENTS (attributes); i++)
 	{
-		const gchar *color = get_color_by_name (scheme, style->background);
+		if (style->mask & attributes[i].mask)
+		{
+			const gchar **attr = G_STRUCT_MEMBER_P (style, attributes[i].offset);
+			const gchar *color = get_color_by_name (scheme, *attr);
 
-		if (color == NULL)
-			/* warning is spit out in get_color_by_name,
-			 * here we make sure style doesn't have NULL color */
-			style->mask &= ~GTK_SOURCE_STYLE_USE_BACKGROUND;
-		else
-			style->background = g_intern_string (color);
-	}
-
-	if (style->mask & GTK_SOURCE_STYLE_USE_FOREGROUND)
-	{
-		const gchar *color = get_color_by_name (scheme, style->foreground);
-
-		if (color == NULL)
-			style->mask &= ~GTK_SOURCE_STYLE_USE_FOREGROUND;
-		else
-			style->foreground = g_intern_string (color);
+			if (color == NULL)
+				/* warning is spit out in get_color_by_name,
+				 * here we make sure style doesn't have NULL color */
+				style->mask &= ~attributes[i].mask;
+			else
+				*attr = g_intern_string (color);
+		}
 	}
 
 	return style;
@@ -402,6 +437,9 @@ fix_style_colors (GtkSourceStyleScheme *scheme,
  * scheme file can have "#red" or "blue", and we want to give out styles
  * which have nice colors suitable for gdk_color_parse(), so that GtkSourceStyle
  * foreground and background properties are the same as GtkTextTag's.
+ * Yet we do need to preserve what we got from file in style schemes,
+ * since there may be child schemes which may redefine colors or something,
+ * so we can't translate colors when loading scheme.
  * So, defined_styles hash has named colors; styles returned with get_style()
  * have real colors.
  */
@@ -466,17 +504,8 @@ gtk_source_style_scheme_set_style (GtkSourceStyleScheme *scheme,
 }
 #endif
 
-/**
- * gtk_source_style_scheme_get_matching_brackets_style:
- * @scheme: a #GtkSourceStyleScheme.
- *
- * Returns: style which corresponds to "bracket-match" name, to use
- * in an editor. It is owned by @scheme and may not be unref'ed.
- *
- * Since: 2.0
- */
 GtkSourceStyle *
-gtk_source_style_scheme_get_matching_brackets_style (GtkSourceStyleScheme *scheme)
+_gtk_source_style_scheme_get_matching_brackets_style (GtkSourceStyleScheme *scheme)
 {
 	g_return_val_if_fail (GTK_IS_SOURCE_STYLE_SCHEME (scheme), NULL);
 	return gtk_source_style_scheme_get_style (scheme, STYLE_BRACKET_MATCH);
@@ -519,19 +548,12 @@ get_color (GtkSourceStyle *style,
 	return FALSE;
 }
 
-/**
- * gtk_source_style_scheme_get_current_line_color:
- * @scheme: a #GtkSourceStyleScheme.
- * @color: a #GdkColor structure to fill.
- *
- * Returns: %TRUE if @scheme has style for current line set, or %FALSE
- * otherwise.
- *
- * Since: 2.0
+/*
+ * Returns TRUE if the style for current-line set in the scheme
  */
 gboolean
-gtk_source_style_scheme_get_current_line_color (GtkSourceStyleScheme *scheme,
-						GdkColor             *color)
+_gtk_source_style_scheme_get_current_line_color (GtkSourceStyleScheme *scheme,
+						 GdkColor             *color)
 {
 	GtkSourceStyle *style;
 
@@ -753,7 +775,7 @@ parse_style (GtkSourceStyleScheme *scheme,
 {
 	GtkSourceStyle *use_style = NULL;
 	GtkSourceStyle *result = NULL;
-	xmlChar *fg = NULL, *bg = NULL;
+	xmlChar *fg = NULL, *bg = NULL, *line_bg = NULL;
 	gchar *style_name = NULL;
 	guint mask = 0;
 	gboolean bold = FALSE;
@@ -801,6 +823,7 @@ parse_style (GtkSourceStyleScheme *scheme,
 
 	fg = xmlGetProp (node, BAD_CAST "foreground");
 	bg = xmlGetProp (node, BAD_CAST "background");
+	line_bg = xmlGetProp (node, BAD_CAST "line-background");
 	get_bool (node, "italic", &mask, GTK_SOURCE_STYLE_USE_ITALIC, &italic);
 	get_bool (node, "bold", &mask, GTK_SOURCE_STYLE_USE_BOLD, &bold);
 	get_bool (node, "underline", &mask, GTK_SOURCE_STYLE_USE_UNDERLINE, &underline);
@@ -808,7 +831,7 @@ parse_style (GtkSourceStyleScheme *scheme,
 
 	if (use_style)
 	{
-		if (fg != NULL || bg != NULL || mask != 0)
+		if (fg != NULL || bg != NULL || line_bg != NULL || mask != 0)
 		{
 			g_set_error (error, ERROR_QUARK, 0,
 				     "in style '%s': style attributes used along with use-style",
@@ -844,6 +867,12 @@ parse_style (GtkSourceStyleScheme *scheme,
 			result->background = g_intern_string ((char*) bg);
 			result->mask |= GTK_SOURCE_STYLE_USE_BACKGROUND;
 		}
+
+		if (line_bg != NULL)
+		{
+			result->line_background = g_intern_string ((char*) line_bg);
+			result->mask |= GTK_SOURCE_STYLE_USE_LINE_BACKGROUND;
+		}
 	}
 
 	*style_p = result;
@@ -911,7 +940,11 @@ parse_style_scheme_child (GtkSourceStyleScheme *scheme,
 	else if (strcmp ((char*) node->name, "author") == 0)
 	{
 		xmlChar *tmp = xmlNodeGetContent (node);
-		scheme->priv->author = g_strdup ((char*) tmp);
+		if (scheme->priv->authors == NULL)
+			scheme->priv->authors = g_ptr_array_new ();
+
+		g_ptr_array_add (scheme->priv->authors, g_strdup ((char*) tmp));
+		
 		xmlFree (tmp);
 	}
 	else if (strcmp ((char*) node->name, "description") == 0)
@@ -995,6 +1028,10 @@ parse_style_scheme_element (GtkSourceStyleScheme *scheme,
 		if (node->type == XML_ELEMENT_NODE)
 			if (!parse_style_scheme_child (scheme, node, error))
 				return;
+				
+	/* NULL-terminate the array of authors */
+	if (scheme->priv->authors != NULL)
+		g_ptr_array_add (scheme->priv->authors, NULL);
 }
 
 /**
@@ -1124,10 +1161,10 @@ _gtk_source_style_scheme_set_parent (GtkSourceStyleScheme *scheme,
 GtkSourceStyleScheme *
 _gtk_source_style_scheme_get_default (void)
 {
-	GtkSourceStyleManager *manager;
+	GtkSourceStyleSchemeManager *manager;
 
-	manager = gtk_source_style_manager_get_default ();
+	manager = gtk_source_style_scheme_manager_get_default ();
 
-	return gtk_source_style_manager_get_scheme (manager,
-						    DEFAULT_STYLE_SCHEME);
+	return gtk_source_style_scheme_manager_get_scheme (manager,
+							   DEFAULT_STYLE_SCHEME);
 }
