@@ -60,24 +60,28 @@ GtkSourceStyleScheme	*_gtk_source_style_scheme_new		(const gchar          *id,
 const gchar             *gtk_source_style_scheme_get_id         (GtkSourceStyleScheme *scheme);
 const gchar             *gtk_source_style_scheme_get_name       (GtkSourceStyleScheme *scheme);
 const gchar             *gtk_source_style_scheme_get_description(GtkSourceStyleScheme *scheme);
+
+G_CONST_RETURN gchar* G_CONST_RETURN *
+			 gtk_source_style_scheme_get_authors	(GtkSourceStyleScheme *scheme);
+
 const gchar             *gtk_source_style_scheme_get_filename	(GtkSourceStyleScheme *scheme);
 
 GtkSourceStyle		*gtk_source_style_scheme_get_style	(GtkSourceStyleScheme *scheme,
 								 const gchar          *style_id);
-GtkSourceStyle		*gtk_source_style_scheme_get_matching_brackets_style
-								(GtkSourceStyleScheme *scheme);
-gboolean		 gtk_source_style_scheme_get_current_line_color
-								(GtkSourceStyleScheme *scheme,
-								 GdkColor             *color);
-
-void			 _gtk_source_style_scheme_apply		(GtkSourceStyleScheme *scheme,
-								 GtkWidget            *widget);
 GtkSourceStyleScheme	*_gtk_source_style_scheme_new_from_file (const gchar          *filename);
 GtkSourceStyleScheme	*_gtk_source_style_scheme_get_default	(void);
 const gchar		*_gtk_source_style_scheme_get_parent_id	(GtkSourceStyleScheme *scheme);
 void			 _gtk_source_style_scheme_set_parent	(GtkSourceStyleScheme *scheme,
 								 GtkSourceStyleScheme *parent_scheme);
 
+/* private */
+void			 _gtk_source_style_scheme_apply		(GtkSourceStyleScheme *scheme,
+								 GtkWidget            *widget);
+GtkSourceStyle		*_gtk_source_style_scheme_get_matching_brackets_style
+								(GtkSourceStyleScheme *scheme);
+gboolean		 _gtk_source_style_scheme_get_current_line_color
+								(GtkSourceStyleScheme *scheme,
+								 GdkColor             *color);
 G_END_DECLS
 
 #endif  /* __GTK_SOURCE_STYLE_SCHEME_H__ */
