@@ -66,9 +66,10 @@
 #define DEFAULT_TAB_WIDTH 		8
 #define MIN_NUMBER_WINDOW_WIDTH		20
 #define MAX_TAB_WIDTH			32
+#define MAX_INDENT_WIDTH		32
 
-#define DEFAULT_MARGIN			80
-#define MAX_MARGIN			200
+#define DEFAULT_RIGHT_MARGIN_POSITION	80
+#define MAX_RIGHT_MARGIN_POSITION	200
 
 /* Signals */
 enum {
@@ -82,11 +83,12 @@ enum {
 	PROP_0,
 	PROP_SHOW_LINE_NUMBERS,
 	PROP_SHOW_LINE_MARKERS,
-	PROP_TABS_WIDTH,
+	PROP_TAB_WIDTH,
+	PROP_INDENT_WIDTH,
 	PROP_AUTO_INDENT,
 	PROP_INSERT_SPACES,
-	PROP_SHOW_MARGIN,
-	PROP_MARGIN,
+	PROP_SHOW_RIGHT_MARGIN,
+	PROP_RIGHT_MARGIN_POSITION,
 	PROP_SMART_HOME_END,
 	PROP_HIGHLIGHT_CURRENT_LINE,
 	PROP_INDENT_ON_TAB
@@ -94,18 +96,20 @@ enum {
 
 struct _GtkSourceViewPrivate
 {
-	guint		 tabs_width;
+	guint		 tab_width;
+	gint		 indent_width;
 	gboolean 	 show_line_numbers;
 	gboolean	 show_line_markers;
 	gboolean	 auto_indent;
 	gboolean	 insert_spaces;
-	gboolean	 show_margin;
 	gboolean	 highlight_current_line;
 	gboolean	 indent_on_tab;
-	guint		 margin;
-	gint             cached_margin_width;
 	GtkSourceSmartHomeEndType smart_home_end;
 
+	gboolean	 show_right_margin;
+	guint		 right_margin_pos;
+	gint             cached_right_margin_pos;
+
 	gboolean	 style_scheme_applied;
 	GtkSourceStyleScheme *style_scheme;
 	GdkGC		*current_line_gc;
@@ -242,16 +246,36 @@ gtk_source_view_class_init (GtkSourceViewClass *klass)
 							       FALSE,
 							       G_PARAM_READWRITE));
 
+	/**
+	 * GtkSourceView:tab-width:
+	 *
+	 * Width of an tab character expressed in number of spaces.
+	 */
 	g_object_class_install_property (object_class,
-					 PROP_TABS_WIDTH,
-					 g_param_spec_uint ("tabs_width",
-							    _("Tabs Width"),
-							    _("Tabs Width"),
+					 PROP_TAB_WIDTH,
+					 g_param_spec_uint ("tab-width",
+							    _("Tab Width"),
+							    _("Width of a tab character expressed in spaces"),
 							    1,
 							    MAX_TAB_WIDTH,
 							    DEFAULT_TAB_WIDTH,
 							    G_PARAM_READWRITE));
 
+	/**
+	 * GtkSourceView:indent-width:
+	 *
+	 * Width of an indentation step expressed in number of spaces.
+	 */
+	g_object_class_install_property (object_class,
+					 PROP_INDENT_WIDTH,
+					 g_param_spec_int ("indent-width",
+							   _("Indent Width"),
+							   _("Number of spaces to use for each step of indent"),
+							   -1,
+							   MAX_INDENT_WIDTH,
+							   -1,
+							   G_PARAM_READWRITE));
+
 	g_object_class_install_property (object_class,
 					 PROP_AUTO_INDENT,
 					 g_param_spec_boolean ("auto_indent",
@@ -268,22 +292,32 @@ gtk_source_view_class_init (GtkSourceViewClass *klass)
 							       FALSE,
 							       G_PARAM_READWRITE));
 
+	/**
+	 * GtkSourceView:show-right-margin:
+	 *
+	 * Whether to display the right margin.
+	 */
 	g_object_class_install_property (object_class,
-					 PROP_SHOW_MARGIN,
-					 g_param_spec_boolean ("show_margin",
+					 PROP_SHOW_RIGHT_MARGIN,
+					 g_param_spec_boolean ("show-right-margin",
 							       _("Show Right Margin"),
 							       _("Whether to display the right margin"),
 							       FALSE,
 							       G_PARAM_READWRITE));
 
+	/**
+	 * GtkSourceView:right-margin-position:
+	 *
+	 * Position of the right margin.
+	 */
 	g_object_class_install_property (object_class,
-					 PROP_MARGIN,
-					 g_param_spec_uint ("margin",
-							    _("Margin position"),
+					 PROP_RIGHT_MARGIN_POSITION,
+					 g_param_spec_uint ("right-margin-position",
+							    _("Right Margin Position"),
 							    _("Position of the right margin"),
 							    1,
-							    MAX_MARGIN,
-							    DEFAULT_MARGIN,
+							    MAX_RIGHT_MARGIN_POSITION,
+							    DEFAULT_RIGHT_MARGIN_POSITION,
 							    G_PARAM_READWRITE));
 
 	/**
@@ -470,9 +504,14 @@ gtk_source_view_set_property (GObject      *object,
 							       g_value_get_boolean (value));
 			break;
 
-		case PROP_TABS_WIDTH:
-			gtk_source_view_set_tabs_width (view,
-							g_value_get_uint (value));
+		case PROP_TAB_WIDTH:
+			gtk_source_view_set_tab_width (view,
+						       g_value_get_uint (value));
+			break;
+
+		case PROP_INDENT_WIDTH:
+			gtk_source_view_set_indent_width (view,
+							  g_value_get_int (value));
 			break;
 
 		case PROP_AUTO_INDENT:
@@ -486,14 +525,14 @@ gtk_source_view_set_property (GObject      *object,
 							g_value_get_boolean (value));
 			break;
 
-		case PROP_SHOW_MARGIN:
-			gtk_source_view_set_show_margin (view,
-							 g_value_get_boolean (value));
+		case PROP_SHOW_RIGHT_MARGIN:
+			gtk_source_view_set_show_right_margin (view,
+							       g_value_get_boolean (value));
 			break;
 
-		case PROP_MARGIN:
-			gtk_source_view_set_margin (view,
-						    g_value_get_uint (value));
+		case PROP_RIGHT_MARGIN_POSITION:
+			gtk_source_view_set_right_margin_position (view,
+								   g_value_get_uint (value));
 			break;
 
 		case PROP_SMART_HOME_END:
@@ -534,41 +573,41 @@ gtk_source_view_get_property (GObject    *object,
 		case PROP_SHOW_LINE_NUMBERS:
 			g_value_set_boolean (value,
 					     gtk_source_view_get_show_line_numbers (view));
-
 			break;
 
 		case PROP_SHOW_LINE_MARKERS:
 			g_value_set_boolean (value,
 					     gtk_source_view_get_show_line_markers (view));
-
 			break;
 
-		case PROP_TABS_WIDTH:
+		case PROP_TAB_WIDTH:
 			g_value_set_uint (value,
-					  gtk_source_view_get_tabs_width (view));
+					  gtk_source_view_get_tab_width (view));
+			break;
+
+		case PROP_INDENT_WIDTH:
+			g_value_set_int (value,
+					 gtk_source_view_get_indent_width (view));
 			break;
 
 		case PROP_AUTO_INDENT:
 			g_value_set_boolean (value,
 					     gtk_source_view_get_auto_indent (view));
-
 			break;
 
 		case PROP_INSERT_SPACES:
 			g_value_set_boolean (value,
 					     gtk_source_view_get_insert_spaces_instead_of_tabs (view));
-
 			break;
 
-		case PROP_SHOW_MARGIN:
+		case PROP_SHOW_RIGHT_MARGIN:
 			g_value_set_boolean (value,
-					     gtk_source_view_get_show_margin (view));
-
+					     gtk_source_view_get_show_right_margin (view));
 			break;
 
-		case PROP_MARGIN:
+		case PROP_RIGHT_MARGIN_POSITION:
 			g_value_set_uint (value,
-					  gtk_source_view_get_margin (view));
+					  gtk_source_view_get_right_margin_position (view));
 			break;
 
 		case PROP_SMART_HOME_END:
@@ -606,11 +645,12 @@ gtk_source_view_init (GtkSourceView *view)
 	view->priv = G_TYPE_INSTANCE_GET_PRIVATE (view, GTK_TYPE_SOURCE_VIEW,
 						  GtkSourceViewPrivate);
 
-	view->priv->tabs_width = DEFAULT_TAB_WIDTH;
-	view->priv->margin = DEFAULT_MARGIN;
-	view->priv->cached_margin_width = -1;
+	view->priv->tab_width = DEFAULT_TAB_WIDTH;
+	view->priv->indent_width = -1;
 	view->priv->indent_on_tab = TRUE;
 	view->priv->smart_home_end = GTK_SOURCE_SMART_HOME_END_DISABLED;
+	view->priv->right_margin_pos = DEFAULT_RIGHT_MARGIN_POSITION;
+	view->priv->cached_right_margin_pos = -1;
 
 	view->priv->pixmap_cache = g_hash_table_new_full (g_str_hash, g_str_equal,
 							  (GDestroyNotify) g_free,
@@ -1560,9 +1600,14 @@ gtk_source_view_expose (GtkWidget      *widget,
 				gc = widget->style->bg_gc[GTK_WIDGET_STATE (widget)];
 
 			if (text_view->hadjustment)
-				margin = gtk_text_view_get_left_margin (text_view) - (int) text_view->hadjustment->value;
+			{
+				margin = gtk_text_view_get_left_margin (text_view) -
+					 (int) text_view->hadjustment->value;
+			}
 			else
+			{
 				margin = gtk_text_view_get_left_margin (text_view);
+			}
 
 			gdk_draw_rectangle (event->window,
 					    gc,
@@ -1579,7 +1624,7 @@ gtk_source_view_expose (GtkWidget      *widget,
 				GTK_WIDGET_CLASS (gtk_source_view_parent_class)->expose_event (widget, event);
 
 		/* Draw the right margin vertical line + overlay. */
-		if (view->priv->show_margin &&
+		if (view->priv->show_right_margin &&
 		    (event->window == gtk_text_view_get_window (text_view, GTK_TEXT_WINDOW_TEXT)))
 		{
 			GdkRectangle visible_rect;
@@ -1594,9 +1639,13 @@ gtk_source_view_expose (GtkWidget      *widget,
 			static GTimer *timer = NULL;
 #endif
 
-			if (view->priv->cached_margin_width < 0)
-				view->priv->cached_margin_width =
-					calculate_real_tab_width (view, view->priv->margin, '_');
+			if (view->priv->cached_right_margin_pos < 0)
+			{
+				view->priv->cached_right_margin_pos =
+					calculate_real_tab_width (view,
+								  view->priv->right_margin_pos,
+								  '_');
+			}
 
 #ifdef ENABLE_PROFILE
 			if (timer == NULL)
@@ -1625,7 +1674,7 @@ gtk_source_view_expose (GtkWidget      *widget,
 			cairo_clip (cr);
 
 			/* Offset with 0.5 is needed for a sharp line. */
-			x = view->priv->cached_margin_width -
+			x = view->priv->cached_right_margin_pos -
 				visible_rect.x + redraw_rect.x + 0.5 +
 				gtk_text_view_get_left_margin (text_view);
 
@@ -1689,7 +1738,8 @@ gtk_source_view_expose (GtkWidget      *widget,
 
 			PROFILE ({
 				g_timer_stop (timer);
-				g_message ("Time to draw the margin: %g (sec * 1000)", g_timer_elapsed (timer, NULL) * 1000);
+				g_message ("Time to draw the margin: %g (sec * 1000)",
+				           g_timer_elapsed (timer, NULL) * 1000);
 			});
 		}
 	}
@@ -1919,30 +1969,13 @@ gtk_source_view_set_show_line_markers (GtkSourceView *view,
 	}
 }
 
-/**
- * gtk_source_view_get_tabs_width:
- * @view: a #GtkSourceView.
- *
- * Returns the width of tabulation in characters.
- *
- * Return value: width of tab.
- **/
-guint
-gtk_source_view_get_tabs_width (GtkSourceView *view)
-{
-	g_return_val_if_fail (view != NULL, FALSE);
-	g_return_val_if_fail (GTK_IS_SOURCE_VIEW (view), FALSE);
-
-	return view->priv->tabs_width;
-}
-
 static gboolean
 set_tab_stops_internal (GtkSourceView *view)
 {
 	PangoTabArray *tab_array;
 	gint real_tab_width;
 
-	real_tab_width = calculate_real_tab_width (view, view->priv->tabs_width, ' ');
+	real_tab_width = calculate_real_tab_width (view, view->priv->tab_width, ' ');
 
 	if (real_tab_width < 0)
 		return FALSE;
@@ -1959,41 +1992,96 @@ set_tab_stops_internal (GtkSourceView *view)
 }
 
 /**
- * gtk_source_view_set_tabs_width:
+ * gtk_source_view_set_tab_width:
  * @view: a #GtkSourceView.
  * @width: width of tab in characters.
  *
  * Sets the width of tabulation in characters.
- *
- **/
+ */
 void
-gtk_source_view_set_tabs_width (GtkSourceView *view,
-				guint          width)
+gtk_source_view_set_tab_width (GtkSourceView *view,
+			       guint          width)
 {
 	guint save_width;
 
 	g_return_if_fail (GTK_SOURCE_VIEW (view));
-	g_return_if_fail (width <= MAX_TAB_WIDTH);
-	g_return_if_fail (width > 0);
+	g_return_if_fail (width > 0 && width <= MAX_TAB_WIDTH);
 
-	if (view->priv->tabs_width == width)
+	if (view->priv->tab_width == width)
 		return;
 
 	gtk_widget_ensure_style (GTK_WIDGET (view));
 
-	save_width = view->priv->tabs_width;
-	view->priv->tabs_width = width;
+	save_width = view->priv->tab_width;
+	view->priv->tab_width = width;
 	if (set_tab_stops_internal (view))
 	{
-		g_object_notify (G_OBJECT (view), "tabs_width");
+		g_object_notify (G_OBJECT (view), "tab-width");
 	}
 	else
 	{
-		g_warning ("Impossible to set tabs width.");
-		view->priv->tabs_width = save_width;
+		g_warning ("Impossible to set tab width.");
+		view->priv->tab_width = save_width;
 	}
 }
 
+/**
+ * gtk_source_view_get_tab_width:
+ * @view: a #GtkSourceView.
+ *
+ * Returns the width of tabulation in characters.
+ *
+ * Return value: width of tab.
+ */
+guint
+gtk_source_view_get_tab_width (GtkSourceView *view)
+{
+	g_return_val_if_fail (view != NULL, FALSE);
+	g_return_val_if_fail (GTK_IS_SOURCE_VIEW (view), FALSE);
+
+	return view->priv->tab_width;
+}
+
+/**
+ * gtk_source_view_set_indent_width:
+ * @view: a #GtkSourceView.
+ * @width: indent width in characters.
+ *
+ * Sets the number of spaces to use for each step of indent.
+ * If @width is -1, the value of the GtkSourceView::tab-width property
+ * will be used.
+ */
+void
+gtk_source_view_set_indent_width (GtkSourceView *view,
+				  gint          width)
+{
+	g_return_if_fail (GTK_SOURCE_VIEW (view));
+	g_return_if_fail ((width == -1) || (width > 0 && width <= MAX_INDENT_WIDTH));
+
+	if (view->priv->indent_width != width)
+	{
+		view->priv->indent_width = width;
+		g_object_notify (G_OBJECT (view), "indent-width");
+	}
+}
+
+/**
+ * gtk_source_view_get_indent_width:
+ * @view: a #GtkSourceView.
+ *
+ * Returns the number of spaces to use for each step of indent.
+ * See gtk_source_view_set_indent_width() for details.
+ *
+ * Return value: indent width.
+ */
+gint
+gtk_source_view_get_indent_width (GtkSourceView *view)
+{
+	g_return_val_if_fail (view != NULL && GTK_IS_SOURCE_VIEW (view), 0);
+
+	return view->priv->indent_width;
+}
+
 /**
  * gtk_source_view_set_marker_pixbuf:
  * @view: a #GtkSourceView.
@@ -2001,7 +2089,7 @@ gtk_source_view_set_tabs_width (GtkSourceView *view,
  * @pixbuf: a #GdkPixbuf.
  *
  * Associates a given @pixbuf with a given @marker_type.
- **/
+ */
 void
 gtk_source_view_set_marker_pixbuf (GtkSourceView *view,
 				   const gchar   *marker_type,
@@ -2052,7 +2140,7 @@ gtk_source_view_set_marker_pixbuf (GtkSourceView *view,
  * Gets the pixbuf which is associated with the given @marker_type.
  *
  * Return value: a #GdkPixbuf if found, or %NULL if not found.
- **/
+ */
 GdkPixbuf *
 gtk_source_view_get_marker_pixbuf (GtkSourceView *view,
 				   const gchar   *marker_type)
@@ -2108,13 +2196,38 @@ compute_indentation (GtkSourceView *view,
 	return gtk_text_iter_get_slice (&start, &end);
 }
 
+static gint
+get_real_indent_width (GtkSourceView *view)
+{
+	return view->priv->indent_width < 0 ?
+	       view->priv->tab_width :
+	       view->priv->indent_width;
+}
+
+static gchar *
+get_indent_string (gint tabs, gint spaces)
+{
+	gchar *str;
+
+	str = g_malloc (tabs + spaces + 1);
+	if (tabs > 0)
+		memset (str, '\t', tabs);
+	if (spaces > 0)
+		memset (str + tabs, ' ', spaces);
+	str[tabs + spaces] = '\0';
+
+	return str;
+}
+
 static void
 indent_lines (GtkSourceView *view, GtkTextIter *start, GtkTextIter *end)
 {
 	GtkTextBuffer *buf;
 	gint start_line, end_line;
-	gint i;
 	gchar *tab_buffer = NULL;
+	gint tabs = 0;
+	gint spaces = 0;
+	gint i;
 
 	buf = gtk_text_view_get_buffer (GTK_TEXT_VIEW (view));
 
@@ -2127,15 +2240,25 @@ indent_lines (GtkSourceView *view, GtkTextIter *start, GtkTextIter *end)
 		end_line--;
 	}
 
-	if (gtk_source_view_get_insert_spaces_instead_of_tabs (view))
+	if (view->priv->insert_spaces)
 	{
-		gint tabs_size;
+		spaces = get_real_indent_width (view);
 
-		tabs_size = view->priv->tabs_width;
-		tab_buffer = g_strnfill (tabs_size, ' ');
+		tab_buffer = g_strnfill (spaces, ' ');
+	}
+	else if (view->priv->indent_width > 0)
+	{
+		gint indent_width;
+
+		indent_width = get_real_indent_width (view);
+		spaces = indent_width % view->priv->tab_width;
+		tabs = indent_width / view->priv->tab_width;		
+
+		tab_buffer = get_indent_string (tabs, spaces);
 	}
 	else
 	{
+		tabs = 1;
 		tab_buffer = g_strdup ("\t");
 	}
 
@@ -2144,14 +2267,52 @@ indent_lines (GtkSourceView *view, GtkTextIter *start, GtkTextIter *end)
 	for (i = start_line; i <= end_line; i++)
 	{
 		GtkTextIter iter;
+		GtkTextIter iter2;
+		gint replaced_spaces = 0;
 
 		gtk_text_buffer_get_iter_at_line (buf, &iter, i);
 
+		/* add spaces always after tabs, to avoid the case
+		 * where "\t" becomes "  \t" with no visual difference */
+		while (gtk_text_iter_get_char (&iter) == '\t')
+		{
+			gtk_text_iter_forward_char (&iter);
+		}
+
 		/* don't add indentation on empty lines */
 		if (gtk_text_iter_ends_line (&iter))
 			continue;
 
-		gtk_text_buffer_insert (buf, &iter, tab_buffer, -1);
+		/* if tabs are allowed try to merge the spaces
+		 * with the tab we are going to insert (if any) */
+		iter2 = iter;
+		while (!view->priv->insert_spaces &&
+		       (gtk_text_iter_get_char (&iter2) == ' ') &&
+			replaced_spaces < view->priv->tab_width)
+		{
+			++replaced_spaces;
+
+			gtk_text_iter_forward_char (&iter2);
+		}
+
+		if (replaced_spaces > 0)
+		{
+			gchar *indent_buf;
+			gint t, s;
+
+			t = tabs + (spaces + replaced_spaces) / view->priv->tab_width;
+			s = (spaces + replaced_spaces) % view->priv->tab_width;
+			indent_buf = get_indent_string (t, s);
+
+			gtk_text_buffer_delete (buf, &iter, &iter2);
+			gtk_text_buffer_insert (buf, &iter, indent_buf, -1);
+
+			g_free (indent_buf);
+		}
+		else
+		{
+			gtk_text_buffer_insert (buf, &iter, tab_buffer, -1);
+		}
 	}
 
 	gtk_text_buffer_end_user_action (buf);
@@ -2167,6 +2328,8 @@ unindent_lines (GtkSourceView *view, GtkTextIter *start, GtkTextIter *end)
 {
 	GtkTextBuffer *buf;
 	gint start_line, end_line;
+	gint tab_width;
+	gint indent_width;
 	gint i;
 
 	buf = gtk_text_view_get_buffer (GTK_TEXT_VIEW (view));
@@ -2180,52 +2343,49 @@ unindent_lines (GtkSourceView *view, GtkTextIter *start, GtkTextIter *end)
 		end_line--;
 	}
 
+	tab_width = view->priv->tab_width;
+	indent_width = get_real_indent_width (view);
+
 	gtk_text_buffer_begin_user_action (buf);
 
 	for (i = start_line; i <= end_line; i++)
 	{
 		GtkTextIter iter, iter2;
+		gint to_delete = 0;
+		gint to_delete_equiv = 0;
 
 		gtk_text_buffer_get_iter_at_line (buf, &iter, i);
 
-		if (gtk_text_iter_get_char (&iter) == '\t')
+		iter2 = iter;
+
+		while (!gtk_text_iter_ends_line (&iter2) &&
+		       to_delete_equiv < indent_width)
 		{
-			iter2 = iter;
+			gunichar c;
+
+			c = gtk_text_iter_get_char (&iter2);
+			if (c == '\t')
+			{
+				to_delete_equiv += tab_width - to_delete_equiv % tab_width;
+				++to_delete;
+			}
+			else if (c == ' ')
+			{
+				++to_delete_equiv;
+				++to_delete;
+			}
+			else
+			{
+				break;
+			}
+
 			gtk_text_iter_forward_char (&iter2);
-			gtk_text_buffer_delete (buf, &iter, &iter2);
 		}
-		else if (gtk_text_iter_get_char (&iter) == ' ')
+
+		if (to_delete > 0)
 		{
-			gint spaces = 0;
-
-			iter2 = iter;
-
-			while (!gtk_text_iter_ends_line (&iter2))
-			{
-				if (gtk_text_iter_get_char (&iter2) == ' ')
-					spaces++;
-				else
-					break;
-
-				gtk_text_iter_forward_char (&iter2);
-			}
-
-			if (spaces > 0)
-			{
-				gint tabs = 0;
-				gint tabs_size = view->priv->tabs_width;
-
-				tabs = spaces / tabs_size;
-				spaces = spaces - (tabs * tabs_size);
-
-				if (spaces == 0)
-					spaces = tabs_size;
-
-				iter2 = iter;
-
-				gtk_text_iter_forward_chars (&iter2, spaces);
-				gtk_text_buffer_delete (buf, &iter, &iter2);
-			}
+			gtk_text_iter_set_line_offset (&iter2, to_delete);
+			gtk_text_buffer_delete (buf, &iter, &iter2);
 		}
 	}
 
@@ -2235,42 +2395,114 @@ unindent_lines (GtkSourceView *view, GtkTextIter *start, GtkTextIter *end)
 					    gtk_text_buffer_get_insert (buf));
 }
 
+static gint
+get_line_offset_in_equivalent_spaces (GtkSourceView *view,
+				      GtkTextIter *iter)
+{
+	GtkTextIter i;
+	gint tab_width;
+	gint n = 0;
+
+	tab_width = view->priv->tab_width;
+
+	i = *iter;
+	gtk_text_iter_set_line_offset (&i, 0);
+
+	while (!gtk_text_iter_equal (&i, iter))
+	{
+		gunichar c;
+
+		c = gtk_text_iter_get_char (&i);
+		if (c == '\t')
+			n += tab_width - n % tab_width;
+		else
+			++n;
+
+		gtk_text_iter_forward_char (&i);
+	}
+
+	return n;
+}
+
 static void
-insert_tab_or_spaces (GtkSourceView *view, GtkTextIter *start, GtkTextIter *end)
+insert_tab_or_spaces (GtkSourceView *view,
+		      GtkTextIter   *start,
+		      GtkTextIter   *end)
 {
 	GtkTextBuffer *buf;
 	gchar *tab_buf;
+	gint cursor_offset = 0;
 
 	if (view->priv->insert_spaces)
 	{
-		gint tabs_size;
+		gint indent_width;
+		gint pos;
+		gint spaces;
+
+		indent_width = get_real_indent_width (view);
+
+		/* CHECK: is this a performance problem? */
+		pos = get_line_offset_in_equivalent_spaces (view, start);
+		spaces = indent_width - pos % indent_width;
+
+		tab_buf = g_strnfill (spaces, ' ');
+	}
+	else if (view->priv->indent_width > 0)
+	{
 		GtkTextIter iter;
-		gint cur_pos;
-		gint tab_pos;
-		gint num_of_equivalent_spaces;
+		gint i;
+		gint tab_width;
+		gint indent_width;
+		gint from;
+		gint to;
+		gint preceding_spaces = 0;
+		gint following_tabs = 0;
+		gint equiv_spaces;
+		gint tabs;
+		gint spaces;
 
-		tabs_size = view->priv->tabs_width;
+		tab_width = view->priv->tab_width;
+		indent_width = get_real_indent_width (view);
 
+		/* CHECK: is this a performance problem? */
+		from = get_line_offset_in_equivalent_spaces (view, start);
+		to = indent_width * (1 + from / indent_width);
+		equiv_spaces = to - from;
+
+		/* extend the selection to include
+		 * preceding spaces so that if indentation
+		 * width < tab width, two conseutive indentation
+		 * width units get compressed into a tab */
 		iter = *start;
-
-		cur_pos = gtk_text_iter_get_line_offset (start);
-		tab_pos = cur_pos;
-
-		/* If there are tabs in the line take them into account */
-		while (tab_pos > 0)
+		for (i = 0; i < tab_width; ++i)
 		{
-			gunichar c;
-
 			gtk_text_iter_backward_char (&iter);
-			c = gtk_text_iter_get_char (&iter);
-			if (c == '\t')
+
+			if (gtk_text_iter_get_char (&iter) == ' ')
+				++preceding_spaces;
+			else
 				break;
-			tab_pos--;
 		}
 
-		num_of_equivalent_spaces = tabs_size - (cur_pos - tab_pos) % tabs_size;
+		gtk_text_iter_backward_chars (start, preceding_spaces);
 
-		tab_buf = g_strnfill (num_of_equivalent_spaces, ' ');
+		/* now also extend the selection to the following tabs
+		 * since we do not want to insert spaces before a tab
+		 * since it may have no visual effect */
+		while (gtk_text_iter_get_char (end) == '\t')
+		{
+			++following_tabs;
+			gtk_text_iter_forward_char (end);
+		}
+
+		tabs = (preceding_spaces + equiv_spaces) / tab_width;
+		spaces = (preceding_spaces + equiv_spaces) % tab_width;
+
+		tab_buf = get_indent_string (tabs + following_tabs, spaces);
+
+		cursor_offset = gtk_text_iter_get_offset (start) +
+			        tabs +
+		                (following_tabs > 0 ? 1 : spaces);
 	}
 	else
 	{
@@ -2280,15 +2512,27 @@ insert_tab_or_spaces (GtkSourceView *view, GtkTextIter *start, GtkTextIter *end)
 	buf = gtk_text_view_get_buffer (GTK_TEXT_VIEW (view));
 
 	gtk_text_buffer_begin_user_action (buf);
+
 	gtk_text_buffer_delete (buf, start, end);
 	gtk_text_buffer_insert (buf, start, tab_buf, -1);
+
+	/* adjust cursor position if needed */
+	if (cursor_offset > 0)
+	{
+		GtkTextIter iter;
+
+		gtk_text_buffer_get_iter_at_offset (buf, &iter, cursor_offset);
+		gtk_text_buffer_place_cursor (buf, &iter);
+	}
+
 	gtk_text_buffer_end_user_action (buf);
 
 	g_free (tab_buf);
 }
 
 static gboolean
-gtk_source_view_key_press_event (GtkWidget *widget, GdkEventKey *event)
+gtk_source_view_key_press_event (GtkWidget   *widget,
+				 GdkEventKey *event)
 {
 	GtkSourceView *view;
 	GtkTextBuffer *buf;
@@ -2331,7 +2575,7 @@ gtk_source_view_key_press_event (GtkWidget *widget, GdkEventKey *event)
 			if (gtk_im_context_filter_keypress (GTK_TEXT_VIEW(view)->im_context, event))
 				return TRUE;
 
-			/* If an input method has inserted some test while handling the key press event,
+			/* If an input method has inserted some text while handling the key press event,
 			 * the cur iterm may be invalid, so get the iter again */
 			gtk_text_buffer_get_iter_at_mark (buf, &cur, mark);
 
@@ -2675,53 +2919,13 @@ view_dnd_drop (GtkTextView *view,
 	}
 }
 
-/**
- * gtk_source_view_get_show_margin:
- * @view: a #GtkSourceView.
- *
- * Returns whether a margin is displayed.
- *
- * Return value: %TRUE if the margin is showed.
- **/
-gboolean
-gtk_source_view_get_show_margin (GtkSourceView *view)
-{
-	g_return_val_if_fail (GTK_IS_SOURCE_VIEW (view), FALSE);
-
-	return view->priv->show_margin;
-}
-
-/**
- * gtk_source_view_set_show_margin:
- * @view: a #GtkSourceView.
- * @show: whether to show a margin.
- *
- * If %TRUE a margin is displayed
- **/
-void
-gtk_source_view_set_show_margin (GtkSourceView *view, gboolean show)
-{
-	g_return_if_fail (GTK_IS_SOURCE_VIEW (view));
-
-	show = (show != FALSE);
-
-	if (view->priv->show_margin == show)
-		return;
-
-	view->priv->show_margin = show;
-
-	gtk_widget_queue_draw (GTK_WIDGET (view));
-
-	g_object_notify (G_OBJECT (view), "show_margin");
-}
-
 /**
  * gtk_source_view_get_highlight_current_line:
  * @view: a #GtkSourceView
  *
  * Returns whether the current line is highlighted
  *
- * Return value: TRUE if the current line is highlighted
+ * Return value: %TRUE if the current line is highlighted.
  **/
 gboolean
 gtk_source_view_get_highlight_current_line (GtkSourceView *view)
@@ -2736,7 +2940,7 @@ gtk_source_view_get_highlight_current_line (GtkSourceView *view)
  * @view: a #GtkSourceView
  * @show: whether to highlight the current line
  *
- * If TRUE the current line is highlighted
+ * If @show is %TRUE the current line is highlighted.
  **/
 void
 gtk_source_view_set_highlight_current_line (GtkSourceView *view, gboolean hl)
@@ -2756,7 +2960,47 @@ gtk_source_view_set_highlight_current_line (GtkSourceView *view, gboolean hl)
 }
 
 /**
- * gtk_source_view_get_margin:
+ * gtk_source_view_get_show_right_margin:
+ * @view: a #GtkSourceView.
+ *
+ * Returns whether a right margin is displayed.
+ *
+ * Return value: %TRUE if the right margin is shown.
+ **/
+gboolean
+gtk_source_view_get_show_right_margin (GtkSourceView *view)
+{
+	g_return_val_if_fail (GTK_IS_SOURCE_VIEW (view), FALSE);
+
+	return view->priv->show_right_margin;
+}
+
+/**
+ * gtk_source_view_set_show_right_margin:
+ * @view: a #GtkSourceView.
+ * @show: whether to show a right margin.
+ *
+ * If %TRUE a right margin is displayed
+ **/
+void
+gtk_source_view_set_show_right_margin (GtkSourceView *view, gboolean show)
+{
+	g_return_if_fail (GTK_IS_SOURCE_VIEW (view));
+
+	show = (show != FALSE);
+
+	if (view->priv->show_right_margin != show)
+	{
+		view->priv->show_right_margin = show;
+
+		gtk_widget_queue_draw (GTK_WIDGET (view));
+
+		g_object_notify (G_OBJECT (view), "show-right-margin");
+	}
+}
+
+/**
+ * gtk_source_view_get_right_margin_position:
  * @view: a #GtkSourceView.
  *
  * Gets the position of the right margin in the given @view.
@@ -2764,38 +3008,37 @@ gtk_source_view_set_highlight_current_line (GtkSourceView *view, gboolean hl)
  * Return value: the position of the right margin.
  **/
 guint
-gtk_source_view_get_margin  (GtkSourceView *view)
+gtk_source_view_get_right_margin_position  (GtkSourceView *view)
 {
-	g_return_val_if_fail (GTK_IS_SOURCE_VIEW (view), DEFAULT_MARGIN);
-
-	return view->priv->margin;
+	g_return_val_if_fail (GTK_IS_SOURCE_VIEW (view),
+			      DEFAULT_RIGHT_MARGIN_POSITION);
 
+	return view->priv->right_margin_pos;
 }
 
 /**
- * gtk_source_view_set_margin:
+ * gtk_source_view_set_right_margin_position:
  * @view: a #GtkSourceView.
  * @margin: the position of the margin to set.
  *
  * Sets the position of the right margin in the given @view.
- *
  **/
 void
-gtk_source_view_set_margin (GtkSourceView *view, guint margin)
+gtk_source_view_set_right_margin_position (GtkSourceView *view, guint pos)
 {
 	g_return_if_fail (GTK_IS_SOURCE_VIEW (view));
-	g_return_if_fail (margin >= 1);
-	g_return_if_fail (margin <= MAX_MARGIN);
+	g_return_if_fail (pos >= 1);
+	g_return_if_fail (pos <= MAX_RIGHT_MARGIN_POSITION);
 
-	if (view->priv->margin == margin)
-		return;
+	if (view->priv->right_margin_pos != pos)
+	{
+		view->priv->right_margin_pos = pos;
+		view->priv->cached_right_margin_pos = -1;
 
-	view->priv->margin = margin;
-	view->priv->cached_margin_width = -1;
+		gtk_widget_queue_draw (GTK_WIDGET (view));
 
-	gtk_widget_queue_draw (GTK_WIDGET (view));
-
-	g_object_notify (G_OBJECT (view), "margin");
+		g_object_notify (G_OBJECT (view), "right-margin-position");
+	}
 }
 
 /**
@@ -2862,8 +3105,9 @@ gtk_source_view_style_set (GtkWidget *widget, GtkStyle *previous_style)
 
 		/* re-set tab stops */
 		set_tab_stops_internal (view);
-		/* make sure the margin width is recalculated on next expose */
-		view->priv->cached_margin_width = -1;
+
+		/* make sure the margin position is recalculated on next expose */
+		view->priv->cached_right_margin_pos = -1;
 	}
 }
 
@@ -2883,7 +3127,7 @@ update_current_line_gc (GtkSourceView *view)
 	}
 
 	if (view->priv->style_scheme &&
-	    gtk_source_style_scheme_get_current_line_color (view->priv->style_scheme, &color))
+	    _gtk_source_style_scheme_get_current_line_color (view->priv->style_scheme, &color))
 	{
 		GdkGCValues values;
 		gdk_colormap_alloc_color (gtk_widget_get_colormap (widget),
