@@ -16,10 +16,9 @@
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *  GNU Library General Public License for more details.
  *
- *  You should have received a copy of the GNU Library General Public
- *  License* along with this program; if not, write to the Free
- *  Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
- *  02111-1307, USA.
+ *  You should have received a copy of the GNU Library General Public License
+ *  along with this program; if not, write to the Free Software
+ *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
 
 #ifndef __GTK_SOURCE_VIEW_H__
@@ -63,15 +62,28 @@ struct _GtkSourceViewClass
 	void (*_gtk_source_reserved1) (void);
 	void (*_gtk_source_reserved2) (void);
 	void (*_gtk_source_reserved3) (void);
+	void (*_gtk_source_reserved4) (void);
 };
 
+/**
+ * GtkSourceSmartHomeEndType:
+ * @GTK_SOURCE_SMART_HOME_END_DISABLED: smart-home-end disabled.
+ * @GTK_SOURCE_SMART_HOME_END_BEFORE: move to the first/last
+ * non-whitespace character on the first press of the HOME/END keys and
+ * to the beginning/end of the line on the second press.
+ * @GTK_SOURCE_SMART_HOME_END_AFTER: move to the beginning/end of the
+ * line on the first press of the HOME/END keys and to the first/last
+ * non-whitespace character on the second press.
+ * @GTK_SOURCE_SMART_HOME_END_ALWAYS: always move to the first/last
+ * non-whitespace character when the HOME/END keys are pressed.
+ **/
 typedef enum
 {
-	GTKSOURCEVIEW_SMART_HOME_END_DISABLED,
-	GTKSOURCEVIEW_SMART_HOME_END_BEFORE,
-	GTKSOURCEVIEW_SMART_HOME_END_AFTER,
-	GTKSOURCEVIEW_SMART_HOME_END_ALWAYS
-} GtkSourceViewSmartHomeEndType;
+	GTK_SOURCE_SMART_HOME_END_DISABLED,
+	GTK_SOURCE_SMART_HOME_END_BEFORE,
+	GTK_SOURCE_SMART_HOME_END_AFTER,
+	GTK_SOURCE_SMART_HOME_END_ALWAYS
+} GtkSourceSmartHomeEndType;
 
 GType		 gtk_source_view_get_type 		(void) G_GNUC_CONST;
 
@@ -88,9 +100,13 @@ void 		 gtk_source_view_set_show_line_markers  (GtkSourceView   *view,
 							 gboolean         show);
 gboolean	 gtk_source_view_get_show_line_markers  (GtkSourceView   *view);
 
-void 		 gtk_source_view_set_tabs_width 	(GtkSourceView   *view,
+void 		 gtk_source_view_set_tab_width          (GtkSourceView   *view,
 							 guint            width);
-guint            gtk_source_view_get_tabs_width         (GtkSourceView   *view);
+guint            gtk_source_view_get_tab_width          (GtkSourceView   *view);
+
+void		 gtk_source_view_set_indent_width 	(GtkSourceView   *view,
+							 gint             width);
+gint		 gtk_source_view_get_indent_width	(GtkSourceView   *view);
 
 void		 gtk_source_view_set_auto_indent 	(GtkSourceView   *view,
 							 gboolean         enable);
@@ -102,10 +118,6 @@ void		 gtk_source_view_set_insert_spaces_instead_of_tabs
 gboolean	 gtk_source_view_get_insert_spaces_instead_of_tabs
 							(GtkSourceView   *view);
 
-void		 gtk_source_view_set_show_margin 	(GtkSourceView   *view,
-							 gboolean         show);
-gboolean 	 gtk_source_view_get_show_margin 	(GtkSourceView   *view);
-
 void		 gtk_source_view_set_indent_on_tab 	(GtkSourceView   *view,
 							 gboolean         enable);
 gboolean	 gtk_source_view_get_indent_on_tab 	(GtkSourceView   *view);
@@ -116,9 +128,15 @@ void		 gtk_source_view_set_highlight_current_line
 gboolean 	 gtk_source_view_get_highlight_current_line
 							(GtkSourceView   *view);
 
-void		 gtk_source_view_set_margin 		(GtkSourceView   *view,
-							 guint            margin);
-guint		 gtk_source_view_get_margin 		(GtkSourceView   *view);
+void		 gtk_source_view_set_show_right_margin 	(GtkSourceView   *view,
+							 gboolean         show);
+gboolean 	 gtk_source_view_get_show_right_margin 	(GtkSourceView   *view);
+
+void		 gtk_source_view_set_right_margin_position
+					 		(GtkSourceView   *view,
+							 guint            pos);
+guint		 gtk_source_view_get_right_margin_position
+					 		(GtkSourceView   *view);
 
 void             gtk_source_view_set_marker_pixbuf      (GtkSourceView   *view,
 							 const gchar     *marker_type,
@@ -126,9 +144,9 @@ void             gtk_source_view_set_marker_pixbuf      (GtkSourceView   *view,
 GdkPixbuf	*gtk_source_view_get_marker_pixbuf      (GtkSourceView   *view,
 				       			 const gchar     *marker_type);
 
-void		 gtk_source_view_set_smart_home_end	(GtkSourceView                 *view,
-							 GtkSourceViewSmartHomeEndType  smart_he);
-GtkSourceViewSmartHomeEndType
+void		 gtk_source_view_set_smart_home_end	(GtkSourceView             *view,
+							 GtkSourceSmartHomeEndType  smart_he);
+GtkSourceSmartHomeEndType
 		 gtk_source_view_get_smart_home_end	(GtkSourceView   *view);
 
 G_END_DECLS
